use strict;

# after the prediction and analysis step was run via SGE, now we have to average the results and add the average to the results file

# there might be more sequences per res file
#### assume there's only one sequence per res file!!!! NO, there are many sequences per file. For each sequence, there is a line in the results file.
# Fixed that on Apr 18, 2008

if ($#ARGV != 2)
{
    print "Usage: $0 <input_dir> <iteration> <header>\n";
    # header is "Training set" or "Validation set", to be written in the results file
    exit;
}

my $input_dir = $ARGV[0];
my $iteration = $ARGV[1];   # at which iteration I am now, so that I can print  out
my $header = $ARGV[2];

my $file;
my $resfile;
my $line;

my @initial_sens = ();
my @initial_ppv = ();

my @new_sens = ();
my @new_ppv = ();

my $initial_avg_sens;
my $initial_avg_ppv;

my $new_avg_sens;
my $new_avg_ppv;

my $initial_std_sens;
my $initial_std_ppv;

my $new_std_sens;
my $new_std_ppv;

my $i;

foreach $file (glob ("$input_dir/res*.txt"))
{
#    print "$file\n";
   
    # if there was an error in the "predict_and_analyse" program, print an error and exit
    if (-z $file)
    {
        print "ERROR!, File $file is empty! An error must have happened!\n";
        exit;
    }
           
    open (FILE, $file) || die ("Cannot open $file");
    while ($line = <FILE>)
    {

        # Initial set sens: 0.697, Initial set ppv: 0.622, New set sens: 0.545, New set ppv: 0.692
        if ($line =~ /Initial set sens: (\S+), Initial set ppv: (\S+), New set sens: (\S+), New set ppv: (\S+)/)
        {
            if ($1 >= 0)
            {   push (@initial_sens, $1);   }
            if ($2 >= 0)
            {   push (@initial_ppv, $2);    }
            if ($3 >= 0)
            {   push (@new_sens, $3);   }
            if ($4 >= 0)
            {   push (@new_ppv, $4);    }
        }

        
#         #Turner: avg_acc: 0.620, avg_sens: 1.000, avg_ppv: 0.423,
#         if ($line =~ /Turner: avg_acc: (\S+), avg_sens: (\S+), avg_ppv: (\S+),/)
#         {
#             push (@turner_acc, $1);
#             if ($2 >= 0)  { push (@turner_sens, $2); }
#             if ($3 >= 0)  { push (@turner_ppv, $3); }
#         }        
#         #Newpar: avg_acc: 0.544, avg_sens: 1.000, avg_ppv: 0.379, f-measure: 0.550
#         if ($line =~ /Newpar: avg_acc: (\S+), avg_sens: (\S+), avg_ppv: (\S+),/)
#         {
#             push (@new_acc, $1);
#             if ($2 >= 0)  { push (@new_sens, $2); }
#             if ($3 >= 0)  { push (@new_ppv, $3);  }
#         }        
    }
    close (FILE);
}

($initial_avg_sens, $initial_std_sens) = get_avg_and_std (@initial_sens);
($initial_avg_ppv, $initial_std_ppv) = get_avg_and_std (@initial_ppv);

($new_avg_sens, $new_std_sens) = get_avg_and_std (@new_sens);
($new_avg_ppv, $new_std_ppv) = get_avg_and_std (@new_ppv);

my $initial_f = 0;
my $new_f = 0;
if ($initial_avg_sens+$initial_avg_ppv != 0)
{
    $initial_f = 2* $initial_avg_sens * $initial_avg_ppv/($initial_avg_sens+$initial_avg_ppv);
}
if ($new_avg_sens+$new_avg_ppv != 0)
{
    $new_f = 2* $new_avg_sens * $new_avg_ppv/($new_avg_sens+$new_avg_ppv);
}

print "------\n";
print "$header:\n";

printf ("Initial: avg_sens: %.3f \\pm %.3f, avg_ppv: %.3f \\pm %.3f, f-measure: %.3f\n", $initial_avg_sens, $initial_std_sens, $initial_avg_ppv, $initial_std_ppv, $initial_f);

printf ("Newpar : avg_sens: %.3f \\pm %.3f, avg_ppv: %.3f \\pm %.3f, f-measure: %.3f\n", $new_avg_sens, $new_std_sens, $new_avg_ppv, $new_std_ppv, $new_f);

##############################

sub get_avg_and_std
{
    # compute the avg and std of all the elements of this array
    my (@array) = @_;
    my $avg;
    my $std;
    my $num = @array."";
    my $sum = 0;
    my $sum2 = 0;
    my $a;
    foreach $a (@array)
    {
        $sum += $a;
        $sum2 += $a*$a;
    }
    # assume $num > 0 , maybe I should check
    $avg = $sum/$num;
    $std = sqrt ($sum2/$num - $avg*$avg);
    return ($avg, $std);
}




