use strict;

# creates 2 files with the specified upper and lower bounds from the given parameter file.

if ($#ARGV != 4)
{
    print "Usage: $0 <training_set> <param_file> <how_much> <lb_output> <ub_output>\n";
    print "The training set is necessary if how_much is custom\n";
    print "Example: $0 turner_parameters_fm363_constrdangles.txt 0.8% tuner_params_lb.txt tuner_params_ub.txt\n";
    print "Example: $0 turner_parameters_fm363_constrdangles.txt 0.8K tuner_params_lb.txt tuner_params_ub.txt\n";
    exit;
}

# check if these are fine:
my $critical_file = "data/critical_parameters_fm363.txt";
my $simple_model = 0;


my $training_set = $ARGV[0];
my $param_file = $ARGV[1];
my $how_much = $ARGV[2];
# $how_much could be custom - added on July 2, 2007.
# or custom2 - added on July 16
my $lb_output = $ARGV[3];
my $ub_output = $ARGV[4];
my $amount;
my $line;
my @pars = ();
my @lb = ();
my @ub = ();
my $par;
my $allowed;
my $perc_or_kcalpmol;
my @counts = ();    # used for how_much = custom
my $count;

if ($how_much =~ /(.+)%/)
{
    $perc_or_kcalpmol = "perc";
    $amount = $1/100;
}
elsif ($how_much =~ /(.+)K/)
{
    $perc_or_kcalpmol = "kcalpmol";
    $amount = $1;
}
elsif ($how_much eq "custom" || $how_much eq "custom2")
{
    $perc_or_kcalpmol = "custom";
    my $command = "./get_real_counts $training_set $critical_file $simple_model > counts.txt";
    system ($command);
    open (COU, "counts.txt") || die ("Cannot open counts.txt");
    while ($line = <COU>)
    {
        if ($line =~ /\S+\s\d\s(\S+)\n/)
        {
            unshift (@counts, $1);
        }
    }
    close (COU);    
}
else
{
    print "<how_much> is not correct. Example: 0.8%, 0.8K or custom\n";
    exit;
}

#print "@counts\n";
open (PAR, $param_file) || die ("Cannot open file $param_file");
while ($line = <PAR>)
{
    if ($line =~ /^(.+)\n/)
    {
        $par = $1;        
        if ($perc_or_kcalpmol eq "perc")
        {
            $allowed = abs ($par*$amount);
        }
        elsif ($perc_or_kcalpmol eq "kcalpmol")
        {
            $allowed = abs ($amount);
        }
        elsif ($perc_or_kcalpmol eq "custom")
        {
            $count = pop(@counts);
            #print "$count ";
            if ($count >= 20)   { $allowed = 10; }
            elsif ($count == 0) { $allowed = 0.1; }
            else                { $allowed = $count/2; }
        }
        else    # custom2
        {
            $count = pop(@counts);
            #print "$count ";
            if ($count >= 20)   { $allowed = 5; }
            elsif ($count == 0) { $allowed = 0.1; }
            else                { $allowed = $count/4; }
        }


        push (@pars, $par);
        push (@lb, $par-$allowed);
        push (@ub, $par+$allowed);
        
    }
}
close (PAR);


# print the current parameters first
# print "[";
# for (my $i=0; $i < $#pars; $i++)
# {
#     print "$pars[$i], ";
# }
# print "$pars[$#pars]]\n";


# print lb into the file, in matlab format
open (LB, ">$lb_output") || die ("Cannot open $lb_output");
for (my $i=0; $i <= $#lb; $i++)
{
    printf LB ("%.2f\n", $lb[$i]);
}
close (LB);

# print ub
open (UB, ">$ub_output") || die ("Cannot open $ub_output");
for (my $i=0; $i <= $#ub; $i++)
{
    printf UB ("%.2f\n", $ub[$i]);
}
close (UB);

