 use strict;

# Pick the best parameter set, as given by the VALIDATION lines. The best training lines are not neccessarily the best validation lines, they can overfit.
# Save the best parameter set
# If the test_file is not none, then measure the accuracy of the best parameter set on the test set.

if ($#ARGV != 3)
{
    print "Usage: $0 <RES_directory> <test_file> <use_sge_instead_of_mpi:0|1> <min_length_sum>\n";
    exit;
}

my $dir = $ARGV[0];
my $test_file = $ARGV[1];
my $use_sge_instead_of_mpi = $ARGV[2];
my $results_file = "$dir/results_final.txt";
my $output_verbose = "$dir/output_verbose.txt";
my $simple_model = 0;
my $min_length_sum = $ARGV[3];
my $line;
my $best_param_set;
my $param_set;
my $f_initial;
my $f_new;
my $best_f_new = 0;
my $num_split_files;
my $set_line;

# first see of there are validation lines. If there are, then pick according to the valdiation lines. Otherwise, pick according to the training lines.

# just out of curiosity, pick the best out of the training lines
open (FILE, $results_file) || die ("Cannot open $results_file");
while ($line = <FILE>)
{    
    if ($line =~ /Num subopt: (\d+)/)   #     Num subopt: 10
    {
        $param_set = $1;
        while ($line = <FILE>)
        {
            if ($line =~ /Training set/)   {   last;   }
        }                      
        $line = <FILE>;     #     Initial: avg_sens: 0.640 \pm 0.240, avg_ppv: 0.581 \pm 0.243, f-measure: 0.609
        if ($line =~ /f-measure: (\d\.\d+)/)
        {
            $f_initial = $1;
        }
        $line = <FILE>;     #     Newpar : avg_sens: 0.605 \pm 0.256, avg_ppv: 0.558 \pm 0.257, f-measure: 0.581
        if ($line =~ /f-measure: (\d\.\d+)/)
        {
            $f_new = $1;
            if ($f_new >= $best_f_new)      # pick the last one, not the first one it might be better
            {
                $best_f_new = $f_new;
                $best_param_set = $param_set;
            }
            #print "f-new = $1\n";
        }
    }
}
close (FILE);
# now add the result to the results_file
open (FILE, ">>$results_file") || die ("Cannot open $results_file");

print FILE "================\n";
print FILE "Best parameter set: $best_param_set\n";
print FILE "Best F-measure on the TRAINING set: $best_f_new, F-measure for Initial: $f_initial\n";
close (FILE);


################
# Now pick the best of the validation set lines

my $validation_lines = 0;
my $initial_train;
my $new_train;
my $bestval_new_train;
my $bestval_initial_train;
$best_f_new = 0;

open (FILE, $results_file) || die ("Cannot open $results_file");
while ($line = <FILE>)
{    
    if ($line =~ /Num subopt: (\d+)/)   #     Num subopt: 10
    {
        $param_set = $1;
        while ($line = <FILE>)
        {
            if ($line =~ /Training set/)   {   last;   }
        }
        $line = <FILE>;
        if ($line =~ /f-measure: (\d\.\d+)/)
        {
            $initial_train = $1;
        }        
        $line = <FILE>;
        if ($line =~ /f-measure: (\d\.\d+)/)
        {
            $new_train = $1;
        }        
        while ($line = <FILE>)
        {
            if ($line =~ /Validation set/)   {   $validation_lines = 1; last;   }
        }                      
        $line = <FILE>;     #     Initial: avg_sens: 0.640 \pm 0.240, avg_ppv: 0.581 \pm 0.243, f-measure: 0.609
        if ($line =~ /f-measure: (\d\.\d+)/)
        {
            $f_initial = $1;
        }
        $line = <FILE>;     #     Newpar : avg_sens: 0.605 \pm 0.256, avg_ppv: 0.558 \pm 0.257, f-measure: 0.581
        if ($line =~ /f-measure: (\d\.\d+)/)
        {
            $f_new = $1;
            if ($f_new >= $best_f_new)      # pick the last one, not the first one it might be better
            {
                $best_f_new = $f_new;
                $best_param_set = $param_set;
                $bestval_initial_train = $initial_train;
                $bestval_new_train = $new_train;
            }
            #print "f-new = $1\n";
        }
    }
}
close (FILE);

if ($validation_lines)
{
    # now add the result to the results_file
    open (FILE, ">>$results_file") || die ("Cannot open $results_file");
    
    print FILE "================\n";
    print FILE "Best parameter set: $best_param_set\n";
    print FILE "Best F-measure on the VALIDATION set : $best_f_new, F-measure for Initial: $f_initial\n";
    print FILE "Corresponding values for TRAINING set: $bestval_new_train, F-measure for Initial: $bestval_initial_train\n";
    print FILE "For table: $best_param_set & $bestval_new_train & $best_f_new\n";
    close (FILE);
    
}





print "VALIDATION: $best_f_new, f turner: $f_initial, parameter set: $best_param_set\n";
print "TRAINING  : $bestval_new_train, f turner: $bestval_initial_train, parameter set: $best_param_set\n";
print "For table: $best_param_set & $bestval_new_train & $best_f_new\n";

# save the best param_set
system ("cp $dir/params_sub-${best_param_set}.txt $dir/final_params.txt");

open (OUTPUT, ">>$output_verbose") || die ("Cannot open $output_verbose");

my $command;

if ($test_file ne "none")
{
    my $num_points = get_num_points ($test_file);
    
    if ($use_sge_instead_of_mpi)
    {
        my $testdir = "$dir/testdir";
        my $new_testdir = "$dir/new_testdir";
    
        print OUTPUT "Now TESTING using SGE. Preparing the test files...\n";
        # first create the testdir from the test file
        $command = "perl split_dataset_into_many_files.pl $min_length_sum $test_file $testdir";
        print "$command\n";
        $num_split_files = `$command`;
        chomp ($num_split_files);        
    
        # then create the new datadir directory
        system ("mkdir $new_testdir");
        print OUTPUT "Submitting SGE array job\n";
        # prepare the sge qsub command to run the prediction and analysis on the cluster
        # fixed a little bug on Oct 24, 2008: added -S /bin/sh, and changes sge_predict_and_analyse_results.sh into sge_validate.sh, because in the case of LAM-CG, the latter applies.
        $command = "cd $dir; qsub -S /bin/sh -cwd -o /dev/null -e /dev/null -t 1000-". (1000+$num_split_files-1)."  sge_validate.sh params_sub-$best_param_set.txt testdir new_testdir";
        print OUTPUT "$command\n";
        # run it via the wrapper which exists only when the jobs are done
        my $start_output = `perl start_sge_job.pl \"$command\"`;
        print OUTPUT "$start_output";
        # once this finished, I have to collect and average the accuracy results, and add it to the results file
        $command = "perl average_analysed_results.pl $new_testdir 0 \"Test set $test_file\" >> $results_file";
        print OUTPUT "$command\n";
        system ($command);
    }
    else
    {
        my $command = "mpirun C ./predict_and_analyse_results_mpi $dir/params_sub-$best_param_set.txt $test_file $dir/prediction_test.txt $num_points $dir/counts_sub-$best_param_set.txt 0 $simple_model 0 none";
        
        print "$command\n";
        
        # I'm just doing a stupid trick to "brush" the output text.
        my @output = `$command`;
        # remove the last line, because it doesn't make sense now any more.
#         for (my $i=0; $i < $#output; $i++)
#         {
#             # replace Training by Testing, because now it's testing
#             $output[$i] =~ s/Training/Testing/;
#             print FILE $output[$i];
#         }
    }
}

close (OUTPUT);



####################################

sub get_num_points
{
    my ($train) = @_;
    my $num = `grep ">" $train | wc -l`;
    chomp ($num);
    return $num;
}
