use strict;

# takes as input a training data set, and creates many files, one per sequence, in the specified directory
# modified on Nov 22, 2007. Each file contains several sequences if they are very short.
# Thus, split the original file in fewer files, such that we don't have to run a separate sge job for each little file

# It's different from the file with the same name in EnergyML by the fact that this one returns the number of files written
#   Also the min is larger because simfold is faster than computing gradients
#   Actually, why don't I take this as parameter

if ($#ARGV != 2)
{
    print "Usage: $0 <min_length_sum> <input_dataset> <output_directory>\n";
    exit;
}

my $min_length_sum = $ARGV[0];   # make sure the sum of lengths of sequences in one file is >= min_length_sum
my $input_dataset = $ARGV[1];
my $output_dir = $ARGV[2];
my $line;
my $file_id = 1000;     # allocate 4 digits, so that they are ordered at the end when I put them back together

my $num_files = 0;

if (! -e $output_dir)
{
    system ("mkdir $output_dir");
}

open (INPUT, "$input_dataset") || die ("Cannot open $input_dataset");

# > SSTRAND_ID=ASE_00005; TYPE=Ribonuclease P RNA; EXT_SOURCE=RNase P Database; EXT_ID=A.eutrophus.ct; ORGANISM=Ralstonia eutrophus
# AAAGCAGGCCAGGCAACCGGUUGUCACGCACCGUUUGCCGCAAGGCGGGCGGGGCGCACAGAAUCCGGCUUAUCGGCCUGCUUUGCUU
# (((((((((((((((((())))))).(((.(((((((((....))))))))).)))................).))))))))))....
# _________________()_____________________________________________________________________
# ((((((((((.((((((())))))).(((.(((((((((....))))))))).)))..................))))))))))....
#

my $length_sum = 0;
my $sequence;
while ($line = <INPUT>)
{
    if ($line =~ /^>/)      # new file
    {
        $num_files++;
        open (OUTPUT, ">$output_dir/file$file_id.txt") || die ("Cannot open $output_dir/file$file_id.txt");
        print OUTPUT $line;
        while ($line = <INPUT>)
        {
            if ($line =~ /^[ACGUTacgut]/)
            {
                $sequence = $line;
                chomp ($sequence);                
                #print "$line";
                #print "length=". length($sequence). "\n";
                $length_sum += length($sequence);
            }
            print OUTPUT $line;
            if ($line =~ /^\n/)     # last line for this file
            {
                if ($length_sum < $min_length_sum)
                {
                    #print OUTPUT "\n";
                }
                else
                {
                    close (OUTPUT);
                    #print "length_sum=$length_sum, file_id=$file_id\n";                    
                    $length_sum = 0;
                    $file_id++;
                    last;
                }
            }
        }
    }
    
}

close (INPUT);

print "$num_files\n";
